/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.message.api;

import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.openide.util.Lookup;
import org.openide.windows.TopComponent;

/**
 * This abstract class intercepts Message events.
 * Implementing classes can handle the message in any way, for example:
 * 
 * - Display one message in a notifier
 * - Display many messages in an inbox
 * - Tweet a certain type of message
 * 
 * 
 * @author Gerben Feenstra
 */
public abstract class MessageEventListener implements GuiEventListener<Message> {
    
    protected MessageService messageService = Lookup.getDefault().lookup(MessageService.class);  
    
    /**
     * Implement this method to indicate which types of messages the subclass accepts. 
     * 
     * 
     * @param message Message to check
     * @return Wether this messages is of the right type
     */
    abstract public boolean listensToMessageType(Message message);
    
    /**
     * This method will be called by the onEvent method and must be implemented by the individual classes.
     * This method gets called only when the checks in {@link #onEvent(GuiEvent<Message> event)} are passed.
     * @param event 
     */
    abstract public void onEvent(MessageEvent event);

    /**
     * Indicates wether the implementing class should persist a received message
     * 
     * @param message Message received
     * @return Wether the implementing class should persist a received message
     */
    abstract public boolean persist(Message message);
    
    /**
     * Catch message GuiEvent. 
     * - Checks if implementing class listens to the type of message received. {@link #listensToMessageType(Message message)}
     * - Checks if implementing class wants messages to be persisted, {@link #persist(Message message)}
     *   If so -> persist
     *   If not -> Add to volatile messages list in Service
     *   
     * Forward the event as a MessageEvent to {@link #onEvent(MessageEvent event)}  (Implemented by subclasses)
     * 
     * This method is final because it should not be re-implemented. Use {@link #onEvent(MessageEvent event)} to implement/override. 
     * 
     * @param event The fired event, containing a message as content.
     */
    @Override
    final public void onEvent(GuiEvent<Message> event) {
        Message message = (Message) event.getContent();

        // Module listens to this messagetype
        if(listensToMessageType(message)){
            if(persist(message)){
                messageService.createOrUpdateMessage(message);
            } 
            else{
                messageService.addMessageToMessageList(message);
            }
            onEvent((MessageEvent) event);
        }
    }
}
